@@ -1,13 +1,15 @@
 # -*- coding: utf-8 -*-
 """
 This module provides a class :class:`Expr` to subclass from in order to
-describe expressions. The value of the class is that it allows straightforward
+describe expressions. The hope was that this class would allow straightforward
 interoperability between python packages handling symbolics (SymPy) and units
-(quantities) as well as working without either of those. The price one has to
-pay to allow for this is a somewhat contrived syntax.
+(quantities) as well as working without either of those. The price for this has
+been a complicated implementation and unnatural syntax.
 
 Note that this module is to be considered an implementation detail, and not
-something that should be relied upon in external code.
+something that should be relied upon in external code. Consider the ``.util._expr``
+module deprecated and aim to use SymPy expressions where possible, and handle units
+separately.
 """
 from __future__ import (absolute_import, division, print_function)
 
@@ -28,7 +30,7 @@ def _implicit_conversion(obj):
     elif isinstance(obj, Expr):
         return obj
     elif isinstance(obj, str):
-        return Symbol(obj)
+        return Symbol(unique_keys=(obj,))
 
     if sympy is not None:
         if isinstance(obj, sympy.Mul):
@@ -44,7 +46,7 @@ def _implicit_conversion(obj):
         elif isinstance(obj, sympy.Float):
             return Constant(float(obj))
         elif isinstance(obj, sympy.Symbol):
-            return Symbol(obj.name)
+            return Symbol(unique_keys=(obj.name,))
 
     raise NotImplementedError(
         "Don't know how to convert %s (of type %s)" % (obj, type(obj)))
@@ -274,8 +276,8 @@ class Expr(object):
 
         if isinstance(res, str):
             res = variables[res]
-        elif isinstance(res, Symbol):
-            res = variables[res.args[0]]
+        # elif isinstance(res, Symbol):
+        #     res = variables[res.unique_keys[0]]
 
         if isinstance(res, Expr) and evaluate:
             return res(variables, backend=backend, **kwargs)
@@ -433,16 +435,17 @@ class Expr(object):
     def __mul__(self, other):
         if other == 1:
             return self
+        if isinstance(other, UnaryWrapper):
+            return NotImplemented  # delegate to UnaryWrapper.__rmul__
         return _MulExpr([self, _implicit_conversion(other)])
 
     def __truediv__(self, other):
         if other == 1:
             return self
+        if isinstance(other, UnaryWrapper):
+            return NotImplemented  # delegate to UnaryWrapper.__rtruediv__
         return _DivExpr([self, _implicit_conversion(other)])
 
-    def __rtruediv(self, other):
-        return _DivExpr([_implicit_conversion(other), self])
-
     def __neg__(self):
         if isinstance(self, _NegExpr):
             return self.args[0]
@@ -467,13 +470,50 @@ class Expr(object):
         return _PowExpr([_implicit_conversion(other), self])
 
 
+class UnaryWrapper(Expr):
+
+    def __checks(self):
+        if self.nargs != 1:
+            raise ValueError("UnaryWrapper can only be used when nargs == 1")
+        if self.unique_keys is not None:
+            raise ValueError("UnaryWrapper can only be used when unique_keys are None")
+
+    def __mul__(self, other):
+        self.__checks()
+        arg, = self.args
+        return self.__class__([_MulExpr([arg, _implicit_conversion(other)])])
+
+    def __truediv__(self, other):
+        if other == 1:
+            return self
+        self.__checks()
+        arg, = self.args
+        return self.__class__([_DivExpr([arg, _implicit_conversion(other)])])
+
+    def __rtruediv__(self, other):
+        self.__checks()
+        arg, = self.args
+        return self.__class__([_DivExpr([_implicit_conversion(other), arg])])
+
+    @classmethod
+    def from_callback(cls, callback, attr='__call__', **kwargs):
+        Wrapper = super().from_callback(callback, attr=attr, **kwargs)
+        return lambda *args, **kw: cls(Wrapper(*args, **kw))
+
+
 class _NegExpr(Expr):
 
+    def _str(self, *args, **kwargs):
+        return "-%s" % args[0]._str(*args, **kwargs)
+
+    def __repr__(self):
+        return super(_NegExpr, self)._str(repr)
+
     def __call__(self, variables, backend=math, **kwargs):
         arg0, = self.all_args(variables, backend=backend, **kwargs)
         return -arg0
 
-    def rate_coeff(self, *args, **kwargs):
+    def rate_coeff(self, *args, **kwargs):   # <--- feature creep into base-class...
         return -self.args[0].rate_coeff(*args, **kwargs),
 
 
@@ -483,6 +523,9 @@ class _BinaryExpr(Expr):
     def _str(self, *args, **kwargs):
         return ("({0} %s {1})" % self._op_str).format(*[arg._str(*args, **kwargs) for arg in self.args])
 
+    def __repr__(self):
+        return super(_BinaryExpr, self)._str(repr)
+
     def __call__(self, variables, backend=math, **kwargs):
         arg0, arg1 = self.all_args(variables, backend=backend, **kwargs)
         return self._op(arg0, arg1)
@@ -530,8 +573,16 @@ class Constant(Expr):
 class Symbol(Expr):
     nargs = 1
 
+    def _str(self, *args, **kwargs):
+        uk, = self.unique_keys
+        return uk
+
+    def __repr__(self):
+        return super(Symbol, self)._str(repr)
+
     def __call__(self, variables, backend=None, **kwargs):
-        return variables[self.args[0]]
+        uk, = self.unique_keys
+        return variables[uk]
 
 
 class Function(Expr):
