@@ -4,16 +4,26 @@ General utilities and exceptions.
 """
 from __future__ import (absolute_import, division, print_function)
 
-from collections import defaultdict, namedtuple, Mapping, OrderedDict
+from collections import defaultdict, namedtuple, OrderedDict
+try:
+    from collections.abc import ItemsView, Mapping
+except ImportError:  # Python 2
+    ItemsView = list
+    from collections import Mapping
 from functools import wraps
 from itertools import product
 import os
+import types
 import warnings
 
 from .. import __url__
 from .deprecation import Deprecation
 
 
+def identity(x):
+    return x
+
+
 class NoConvergence(Exception):
     pass
 
@@ -113,7 +123,7 @@ class AttributeContainer(object):
 
     Parameters
     ----------
-    d : dict
+    \\*\\*kwargs : dictionary
 
     Examples
     --------
@@ -135,6 +145,13 @@ class AttributeContainer(object):
         return self.__dict__.copy()
 
 
+class AttrDict(dict):
+    """ Subclass of dict with attribute access to keys """
+    def __init__(self, *args, **kwargs):
+        super(AttrDict, self).__init__(*args, **kwargs)
+        self.__dict__ = self
+
+
 class defaultkeydict(defaultdict):
     """ defaultdict where default_factory should have the signature key -> value
 
@@ -176,8 +193,8 @@ def defaultnamedtuple(typename, field_names, defaults=()):
     >>> Body.__doc__
     'Body(x, y, z, density)'
     >>> b = Body(10, z=3, y=5)
-    >>> b._asdict()
-    OrderedDict([('x', 10), ('y', 5), ('z', 3), ('density', 1.0)])
+    >>> b._asdict() == dict(x=10, y=5, z=3, density=1.0)
+    True
 
     Returns
     -------
@@ -195,7 +212,8 @@ def defaultnamedtuple(typename, field_names, defaults=()):
     return Tuple
 
 
-def multi_indexed_cases(od):
+def multi_indexed_cases(od, *, dict_=OrderedDict, apply_keys=None, apply_values=None,
+                        apply_return=list, named_index=False):
     """ Returns a list of length-2 tuples
 
     Each tuple consist of a multi-index (tuple of integers) and a dictionary.
@@ -203,8 +221,18 @@ def multi_indexed_cases(od):
     Parameters
     ----------
     od : OrderedDict
-        Maps each key to a number of values. ``list`` and ``tuple`` instances
-        are converted to ``OrderedDict``.
+        Maps each key to a number of values. Instances of ``list``, ``tuple``,
+        ``types.GeneratorType``, ``collections.abc.ItemsView`` are converted to ``OrderedDict``.
+    dict_ : type, optional
+        Used in the result (see ``Returns``).
+    apply_keys : callable, optional
+        Transformation of keys.
+    apply_values : callable, optional
+        Transformation of values.
+    apply_return : callable, optional
+        Applied on return value. ``None`` for generator.
+    named_index : bool
+        Tuple of indices will be a ``namedtuple`` (requires all keys to be ``str``).
 
     Examples
     --------
@@ -217,18 +245,38 @@ def multi_indexed_cases(od):
     (0, 0)
     >>> case_kws[0] == {'a': 1, 'b': False}
     True
+    >>> d = {'a': 'foo bar'.split(), 'b': 'baz qux'.split()}
+    >>> from chempy.util.pyutil import AttrDict
+    >>> for nidx, case in multi_indexed_cases(d, dict_=AttrDict, named_index=True):
+    ...     if case.a == 'bar' and case.b == 'baz':
+    ...         print("{} {}".format(nidx.a, nidx.b))
+    ...
+    1 0
+
 
     Returns
     -------
-    List of length-2 tuples, each consisting of one tuple of inidices and one dictionary.
+    List of length-2 tuples, each consisting of one tuple of indices and one dictionary (of type ``dict_``).
 
     """
-    if isinstance(od, (list, tuple)):
-        od = OrderedDict(od)
-    if not isinstance(od, OrderedDict):
-        raise NotImplementedError("Expected an OrderedDcit")
-    keys, values = map(list, [od.keys(), od.values()])
-    return [(mi, {k: v[i] for k, v, i in zip(keys, values, mi)}) for mi in product(*map(range, map(len, values)))]
+    if isinstance(od, OrderedDict):
+        pass
+    else:
+        if hasattr(od, 'items'):
+            od = od.items()
+
+        if isinstance(od, (list, tuple, types.GeneratorType, ItemsView)):
+            od = OrderedDict(od)
+        else:
+            raise NotImplementedError("Expected an OrderedDict")
+
+    keys, values = tuple(map(apply_keys or identity, od.keys())), tuple(od.values())
+    MultiIndex = namedtuple('MultiIndex', keys) if named_index else lambda *args: tuple(args)
+    _generator = ((MultiIndex(*mi), dict_([
+        (k, (apply_values or identity)(v[i])) for
+        k, v, i in zip(keys, values, mi)
+    ])) for mi in product(*map(range, map(len, values))))
+    return (apply_return or identity)(_generator)
 
 
 def memoize(max_nargs=0):
