@@ -6,6 +6,7 @@ Utility functions related to stoichiometry.
 from __future__ import (absolute_import, division, print_function)
 
 import numpy as np
+from chempy.units import unit_of, to_unitless
 
 
 def get_coeff_mtx(substances, stoichs):
@@ -15,10 +16,10 @@ def get_coeff_mtx(substances, stoichs):
 
     Parameters
     ----------
-    substances: sequence of keys in stoichs dict pairs
-    stoichs: sequence of pairs of dicts
-        pairs of reactant and product dicts mapping substance keys
-        to stoichiometric coefficients (integers)
+    substances : sequence of keys in stoichs dict pairs
+    stoichs : sequence of pairs of dicts
+        Pairs of reactant and product dicts mapping substance keys
+        to stoichiometric coefficients (integers).
 
     Returns
     -------
@@ -32,7 +33,7 @@ def get_coeff_mtx(substances, stoichs):
     return A
 
 
-def decompose_yields(yields, stoichs, atol=1e-10):
+def decompose_yields(yields, rxns, atol=1e-10):
     """ Decomposes yields into mass-action reactions
 
     This function offers a way to express a reaction with non-integer
@@ -46,14 +47,23 @@ def decompose_yields(yields, stoichs, atol=1e-10):
 
     Parameters
     ----------
-    yields: OrderedDict
-        specie names as keys and yields as values
-    stoichs: iterable dictionary pairs
-        dict keys must match those of `yields`each pair
+    yields : OrderedDict
+        Specie names as keys and yields as values.
+    rxns : iterable :class:`Reaction` instances
+        Dict keys must match those of ``yields`` each pair
         of dictionaries gives stoichiometry
-        (1st is reactant, 2nd is products)
-    atol: float
-        absolute tolerance for residuals
+        (1st is reactant, 2nd is products).
+    atol : float
+        Absolute tolerance for residuals.
+
+
+    Examples
+    --------
+    >>> from chempy import Reaction
+    >>> h2a = Reaction({'H2O': 1}, {'H2': 1, 'O': 1})
+    >>> h2b = Reaction({'H2O': 1}, {'H2': 1, 'H2O2': 1}, inact_reac={'H2O': 1})
+    >>> decompose_yields({'H2': 3, 'O': 2, 'H2O2': 1}, [h2a, h2b])
+    array([2., 1.])
 
     Raises
     ------
@@ -67,19 +77,19 @@ def decompose_yields(yields, stoichs, atol=1e-10):
     1-dimensional array of effective rate coefficients.
 
     """
+    from chempy import ReactionSystem
     # Sanity check:
-    for ys in yields.keys():
-        present = False
-        for reac, prod in stoichs:
-            if ys in reac or ys in prod:
-                present = True
-        assert present
-
-    sbstncs = yields.keys()
-    y = np.array(list(yields.values()))
-    A = get_coeff_mtx(sbstncs, stoichs)
-    k, residuals, rank, s = np.linalg.lstsq(A, y)
+    rxn_keys = set.union(*(rxn.keys() for rxn in rxns))
+    for key in yields.keys():
+        if key not in rxn_keys:
+            raise ValueError("Substance key: %s not in reactions" % key)
+    rsys = ReactionSystem(rxns, rxn_keys)
+    A = rsys.net_stoichs(yields.keys())
+    b = list(yields.values())
+    unit = unit_of(b[0])
+    x, residuals, rank, s = np.linalg.lstsq(
+        np.asarray(A.T, dtype=np.float64), to_unitless(b, unit), rcond=2e-16*max(A.shape))
     if len(residuals) > 0:
         if np.any(residuals > atol):
             raise ValueError("atol not satisfied")
-    return k
+    return x*unit
